package org.apache.bcel.generic;

import org.apache.bcel.util.ByteSequence;
import org.apache.bcel.classfile.Constant;
import org.apache.bcel.classfile.ConstantClass;
import org.apache.bcel.classfile.ConstantInteger;
import org.apache.bcel.classfile.ConstantFloat;
import org.apache.bcel.classfile.ConstantUtf8;
import org.apache.bcel.classfile.ConstantString;
import org.apache.bcel.ExceptionConst;
import java.io.IOException;
import java.io.DataOutputStream;

public class LDC extends CPInstruction implements PushInstruction, ExceptionThrower
{
    LDC() {
    }
    
    public LDC(final int index) {
        super((short)19, index);
        this.setSize();
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitStackProducer(this);
        v.visitPushInstruction(this);
        v.visitExceptionThrower(this);
        v.visitTypedInstruction(this);
        v.visitCPInstruction(this);
        v.visitLDC(this);
    }
    
    @Override
    public void dump(final DataOutputStream out) throws IOException {
        out.writeByte(super.getOpcode());
        if (super.getLength() == 2) {
            out.writeByte(super.getIndex());
        }
        else {
            out.writeShort(super.getIndex());
        }
    }
    
    @Override
    public Class<?>[] getExceptions() {
        return ExceptionConst.createExceptions(ExceptionConst.EXCS.EXCS_STRING_RESOLUTION, (Class<?>[])new Class[0]);
    }
    
    @Override
    public Type getType(final ConstantPoolGen cpg) {
        switch (cpg.getConstantPool().getConstant(super.getIndex()).getTag()) {
            case 8: {
                return Type.STRING;
            }
            case 4: {
                return Type.FLOAT;
            }
            case 3: {
                return Type.INT;
            }
            case 7: {
                return Type.CLASS;
            }
            default: {
                throw new IllegalArgumentException("Unknown or invalid constant type at " + super.getIndex());
            }
        }
    }
    
    public Object getValue(final ConstantPoolGen cpg) {
        Constant c = cpg.getConstantPool().getConstant(super.getIndex());
        switch (c.getTag()) {
            case 8: {
                final int i = ((ConstantString)c).getStringIndex();
                c = cpg.getConstantPool().getConstant(i);
                return ((ConstantUtf8)c).getBytes();
            }
            case 4: {
                return ((ConstantFloat)c).getBytes();
            }
            case 3: {
                return ((ConstantInteger)c).getBytes();
            }
            case 7: {
                final int nameIndex = ((ConstantClass)c).getNameIndex();
                c = cpg.getConstantPool().getConstant(nameIndex);
                return new ObjectType(((ConstantUtf8)c).getBytes());
            }
            default: {
                throw new IllegalArgumentException("Unknown or invalid constant type at " + super.getIndex());
            }
        }
    }
    
    @Override
    protected void initFromFile(final ByteSequence bytes, final boolean wide) throws IOException {
        super.setLength(2);
        super.setIndex(bytes.readUnsignedByte());
    }
    
    @Override
    public final void setIndex(final int index) {
        super.setIndex(index);
        this.setSize();
    }
    
    protected final void setSize() {
        if (super.getIndex() <= 255) {
            super.setOpcode((short)18);
            super.setLength(2);
        }
        else {
            super.setOpcode((short)19);
            super.setLength(3);
        }
    }
}
